@@ -1,24 +1,19 @@
 // This module defines the "official" high-level API of the Qt port of
 // Scintilla.
 //
-// Copyright (c) 2014 Riverbank Computing Limited <info@riverbankcomputing.com>
+// Copyright (c) 2017 Riverbank Computing Limited <info@riverbankcomputing.com>
 // 
 // This file is part of QScintilla.
 // 
-// This file may be used under the terms of the GNU General Public
-// License versions 2.0 or 3.0 as published by the Free Software
-// Foundation and appearing in the files LICENSE.GPL2 and LICENSE.GPL3
-// included in the packaging of this file.  Alternatively you may (at
-// your option) use any later version of the GNU General Public
-// License if such license has been publicly approved by Riverbank
-// Computing Limited (or its successors, if any) and the KDE Free Qt
-// Foundation. In addition, as a special exception, Riverbank gives you
-// certain additional rights. These rights are described in the Riverbank
-// GPL Exception version 1.1, which can be found in the file
-// GPL_EXCEPTION.txt in this package.
+// This file may be used under the terms of the GNU General Public License
+// version 3.0 as published by the Free Software Foundation and appearing in
+// the file LICENSE included in the packaging of this file.  Please review the
+// following information to ensure the GNU General Public License version 3.0
+// requirements will be met: http://www.gnu.org/copyleft/gpl.html.
 // 
-// If you are unsure which license is appropriate for your use, please
-// contact the sales department at sales@riverbankcomputing.com.
+// If you do not wish to use this file under the terms of the GPL version 3.0
+// then you may purchase a commercial license.  For more information contact
+// info@riverbankcomputing.com.
 // 
 // This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
 // WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
@@ -27,16 +22,11 @@
 #ifndef QSCISCINTILLA_H
 #define QSCISCINTILLA_H
 
-#ifdef __APPLE__
-extern "C++" {
-#endif
-
-#include <qobject.h>
-#include <qstringlist.h>
-
 #include <QByteArray>
 #include <QList>
+#include <QObject>
 #include <QPointer>
+#include <QStringList>
 
 #include <Qsci/qsciglobal.h>
 #include <Qsci/qscicommand.h>
@@ -98,7 +88,10 @@ public:
         AnnotationStandard = ANNOTATION_STANDARD,
 
         //! Annotations are surrounded by a box.
-        AnnotationBoxed = ANNOTATION_BOXED
+        AnnotationBoxed = ANNOTATION_BOXED,
+
+        //! Annotations are indented to match the text.
+        AnnotationIndented = ANNOTATION_INDENTED,
     };
 
     //! This enum defines the behavior if an auto-completion list contains a
@@ -193,7 +186,11 @@ public:
         //! The background color of characters after the column limit is
         //! changed to the color set by setEdgeColor().  This is recommended
         //! for proportional fonts.
-        EdgeBackground = EDGE_BACKGROUND
+        EdgeBackground = EDGE_BACKGROUND,
+
+        //! Multiple vertical lines are drawn at the columns defined by
+        //! multiple calls to addEdgeColumn().
+        EdgeMultipleLines = EDGE_MULTILINE,
     };
 
     //! This enum defines the different end-of-line modes.
@@ -259,9 +256,16 @@ public:
         RoundBoxIndicator = INDIC_ROUNDBOX,
 
         //! A rectangle around the text with the interior usually more
-        //! transparent than the border.
+        //! transparent than the border.  It does not colour the top pixel of
+        //! the line so that indicators on contiguous lines are visually
+        //! distinct and disconnected.
         StraightBoxIndicator = INDIC_STRAIGHTBOX,
 
+        //! A rectangle around the text with the interior usually more
+        //! transparent than the border.  Unlike StraightBoxIndicator it covers
+        //! the entire character area.
+        FullBoxIndicator = INDIC_FULLBOX,
+
         //! A dashed underline.
         DashesIndicator = INDIC_DASH,
 
@@ -280,9 +284,24 @@ public:
         //! but may be of lower quality.
         SquigglePixmapIndicator = INDIC_SQUIGGLEPIXMAP,
 
-        //! A thick underline typically used during Asian language input
-        //! composition.
+        //! A thick underline typically used for the target during Asian
+        //! language input composition.
         ThickCompositionIndicator = INDIC_COMPOSITIONTHICK,
+
+        //! A thin underline typically used for non-target ranges during Asian
+        //! language input composition.
+        ThinCompositionIndicator = INDIC_COMPOSITIONTHIN,
+
+        //! The color of the text is set to the color of the indicator's
+        //! foreground.
+        TextColorIndicator = INDIC_TEXTFORE,
+
+        //! A triangle below the start of the indicator range.
+        TriangleIndicator = INDIC_POINT,
+
+        //! A triangle below the centre of the first character in the indicator
+        //! range.
+        TriangleCharacterIndicator = INDIC_POINTCHARACTER,
     };
 
     //! This enum defines the different margin options.
@@ -315,7 +334,11 @@ public:
         TextMargin = SC_MARGIN_TEXT,
 
         //! The margin contains right justified styled text.
-        TextMarginRightJustified = SC_MARGIN_RTEXT
+        TextMarginRightJustified = SC_MARGIN_RTEXT,
+
+        //! The margin contains symbols and uses the color set by
+        //! setMarginBackgroundColor() as its background color.
+        SymbolMarginColor = SC_MARGIN_COLOUR,
     };
 
     //! This enum defines the different pre-defined marker symbols.
@@ -410,7 +433,20 @@ public:
 
         //! No symbol is drawn but the line is drawn underlined using the
         //! marker's background color.
-        Underline = SC_MARK_UNDERLINE
+        Underline = SC_MARK_UNDERLINE,
+
+        //! A bookmark.
+        Bookmark = SC_MARK_BOOKMARK,
+    };
+
+    //! This enum defines how tab characters are drawn when whitespace is
+    //! visible.
+    enum TabDrawMode {
+        //! An arrow stretching to the tab stop.
+        TabLongArrow = SCTD_LONGARROW,
+
+        //! A horizontal line stretching to the tab stop.
+        TabStrikeOut = SCTD_STRIKEOUT,
     };
 
     //! This enum defines the different whitespace visibility modes.  When
@@ -424,7 +460,10 @@ public:
         WsVisible = SCWS_VISIBLEALWAYS,
 
         //! Whitespace is visible after the whitespace used for indentation.
-        WsVisibleAfterIndent = SCWS_VISIBLEAFTERINDENT
+        WsVisibleAfterIndent = SCWS_VISIBLEAFTERINDENT,
+
+        //! Whitespace used for indentation is visible.
+        WsVisibleOnlyInIndent = SCWS_VISIBLEONLYININDENT,
     };
 
     //! This enum defines the different line wrap modes.
@@ -436,7 +475,10 @@ public:
         WrapWord = SC_WRAP_WORD,
 
         //! Lines are wrapped at character boundaries.
-        WrapCharacter = SC_WRAP_CHAR
+        WrapCharacter = SC_WRAP_CHAR,
+
+        //! Lines are wrapped at whitespace boundaries.
+        WrapWhitespace = SC_WRAP_WHITESPACE,
     };
 
     //! This enum defines the different line wrap visual flags.
@@ -577,6 +619,12 @@ public:
     //! \sa setBraceMatching()
     BraceMatch braceMatching() const {return braceMode;}
 
+    //! Returns the encoded text between positions \a start and \a end.  This
+    //! is typically used by QsciLexerCustom::styleText().
+    //!
+    //! \sa text()
+    QByteArray bytes(int start, int end) const;
+
     //! Returns the current call tip position.
     //!
     //! \sa setCallTipsPosition()
@@ -916,6 +964,11 @@ public:
     //! \sa setLexer()
     QsciLexer *lexer() const;
 
+    //! Returns the background color of margin \a margin.
+    //!
+    //! \sa setMarginBackgroundColor()
+    QColor marginBackgroundColor(int margin) const;
+
     //! Returns true if line numbers are enabled for margin \a margin.
     //!
     //! \sa setMarginLineNumbers(), marginType(), SCI_GETMARGINTYPEN
@@ -946,6 +999,11 @@ public:
     //! \sa setMarginWidth(), SCI_GETMARGINWIDTHN
     int marginWidth(int margin) const;
 
+    //! Returns the number of margins.
+    //!
+    //! \sa setMargins()
+    int margins() const;
+
     //! Define a type of marker using the symbol \a sym with the marker number
     //! \a markerNumber.  If \a markerNumber is -1 then the marker number is
     //! automatically allocated.  The marker number is returned or -1 if too
@@ -1087,6 +1145,26 @@ public:
     //! \sa setFoldMarginColors()
     void resetFoldMarginColors();
 
+    //! Resets the background colour of an active hotspot area to the default.
+    //!
+    //! \sa setHotspotBackgroundColor(), resetHotspotForegroundColor()
+    void resetHotspotBackgroundColor();
+
+    //! Resets the foreground colour of an active hotspot area to the default.
+    //!
+    //! \sa setHotspotForegroundColor(), resetHotspotBackgroundColor()
+    void resetHotspotForegroundColor();
+
+    //! Gets the assumed document width in pixels.
+    //!
+    //! \sa setScrollWidth(), setScrollWidthTracking()
+    int scrollWidth() const;
+
+    //! Returns true if scroll width tracking is enabled.
+    //!
+    //! \sa scrollWidth(), setScrollWidthTracking()
+    bool scrollWidthTracking() const;
+
     //! The fold margin may be drawn as a one pixel sized checkerboard pattern
     //! of two colours, \a fore and \a back.
     //!
@@ -1166,6 +1244,17 @@ public:
     //! \sa document()
     void setDocument(const QsciDocument &document);
 
+    //! Add \a colnr to the columns which are displayed with a vertical line.
+    //! The edge mode must be set to EdgeMultipleLines.
+    //!
+    //! \sa clearEdgeColumns()
+    void addEdgeColumn(int colnr, const QColor &col);
+
+    //! Remove any columns added by previous calls to addEdgeColumn().
+    //!
+    //! \sa addEdgeColumn()
+    void clearEdgeColumns();
+
     //! Set the color of the marker used to show that a line has exceeded the
     //! length set by setEdgeColumn().
     //!
@@ -1200,11 +1289,26 @@ public:
     //! If \a indicatorNumber is -1 then the colour of all indicators is set.
     void setIndicatorForegroundColor(const QColor &col, int indicatorNumber = -1);
 
+    //! Set the foreground colour of indicator \a indicatorNumber to \a col
+    //! when the mouse is over it or the caret moved into it.  If
+    //! \a indicatorNumber is -1 then the colour of all indicators is set.
+    void setIndicatorHoverForegroundColor(const QColor &col, int indicatorNumber = -1);
+
+    //! Set the style of indicator \a indicatorNumber to \a style when the
+    //! mouse is over it or the caret moved into it.  If \a indicatorNumber is
+    //! -1 then the style of all indicators is set.
+    void setIndicatorHoverStyle(IndicatorStyle style, int indicatorNumber = -1);
+
     //! Set the outline colour of indicator \a indicatorNumber to \a col.
     //! If \a indicatorNumber is -1 then the colour of all indicators is set.
     //! At the moment only the alpha value of the colour has any affect.
     void setIndicatorOutlineColor(const QColor &col, int indicatorNumber = -1);
 
+    //! Sets the background color of margin \a margin to \a col.
+    //!
+    //! \sa marginBackgroundColor()
+    void setMarginBackgroundColor(int margin, const QColor &col);
+
     //! Set the margin options to \a options.
     //!
     //! \sa marginOptions(), MoNone, MoSublineSelect.
@@ -1234,6 +1338,11 @@ public:
     //! then all margin text is removed.
     void clearMarginText(int line = -1);
 
+    //! Set the number of margins to \a margins.
+    //!
+    //! \sa margins()
+    void setMargins(int margins);
+
     //! Set the background colour, including the alpha component, of marker
     //! \a markerNumber to \a col.  If \a markerNumber is -1 then the colour of
     //! all markers is set.  The default is white.
@@ -1271,6 +1380,29 @@ public:
     //! \sa setMatchedBraceIndicator()
     void resetMatchedBraceIndicator();
 
+    //! For performance, QScintilla does not measure the display width of the
+    //! document to determine the properties of the horizontal scroll bar.
+    //! Instead, an assumed width is used.  This sets the document width in
+    //! pixels assumed by QScintilla to \a pixelWidth.  The default value is
+    //! 2000.
+    //!
+    //! \sa scrollWidth(), setScrollWidthTracking()
+    void setScrollWidth(int pixelWidth);
+
+    //! If scroll width tracking is enabled then the scroll width is adjusted
+    //! to ensure that all of the lines currently displayed can be completely
+    //! scrolled.  This mode never adjusts the scroll width to be narrower.
+    //! This sets the scroll width tracking to \a enabled.
+    //!
+    //! \sa setScrollWidth(), scrollWidthTracking()
+    void setScrollWidthTracking(bool enabled);
+
+    //! Sets the mode used to draw tab characters when whitespace is visible to
+    //! \a mode.  The default is to use an arrow.
+    //!
+    //! \sa tabDrawMode()
+    void setTabDrawMode(TabDrawMode mode);
+
     //! Set the background colour used to display unmatched braces to \a col.
     //! It is ignored if an indicator is being used.  The default is white.
     //!
@@ -1314,6 +1446,24 @@ public:
     //! \sa setSelectionToEol()
     bool selectionToEol() const;
 
+    //! Sets the background colour of an active hotspot area to \a col.
+    //!
+    //! \sa resetHotspotBackgroundColor(), setHotspotForegroundColor()
+    void setHotspotBackgroundColor(const QColor &col);
+
+    //! Sets the foreground colour of an active hotspot area to \a col.
+    //!
+    //! \sa resetHotspotForegroundColor(), setHotspotBackgroundColor()
+    void setHotspotForegroundColor(const QColor &col);
+
+    //! Enables or disables, according to \a enable, the underlining of an
+    //! active hotspot area.  The default is false.
+    void setHotspotUnderline(bool enable);
+
+    //! Enables or disables, according to \a enable, the wrapping of a hotspot
+    //! area to following lines.  The default is true.
+    void setHotspotWrap(bool enable);
+
     //! Sets whether or not the selection is drawn up to the right hand border.
     //! \a filled is set if the selection is drawn to the border.
     //!
@@ -1370,6 +1520,12 @@ public:
     //! The standard command set is returned.
     QsciCommandSet *standardCommands() const {return stdCmds;}
 
+    //! Returns the mode used to draw tab characters when whitespace is
+    //! visible.
+    //!
+    //! \sa setTabDrawMode()
+    TabDrawMode tabDrawMode() const;
+
     //! Returns true if the tab key indents a line instead of inserting a tab
     //! character.  The default is true.
     //!
@@ -1393,6 +1549,14 @@ public:
     //! \sa setText()
     QString text(int line) const;
 
+    //! \overload
+    //!
+    //! Returns the text between positions \a start and \a end.  This is
+    //! typically used by QsciLexerCustom::styleText().
+    //!
+    //! \sa bytes(), setText()
+    QString text(int start, int end) const;
+
     //! Returns the height in pixels of the text in line number \a linenr.
     int textHeight(int linenr) const;
 
@@ -1917,6 +2081,15 @@ signals:
     //! \sa marginSensitivity(), setMarginSensitivity()
     void marginClicked(int margin, int line, Qt::KeyboardModifiers state);
 
+    //! This signal is emitted whenever the user right-clicks on a sensitive
+    //! margin.  \a margin is the margin.  \a line is the number of the line
+    //! where the user clicked.  \a state is the state of the modifier keys
+    //! (Qt::ShiftModifier, Qt::ControlModifier, Qt::AltModifer and
+    //! Qt::MetaModifier) when the user clicked.
+    //!
+    //! \sa marginSensitivity(), setMarginSensitivity()
+    void marginRightClicked(int margin, int line, Qt::KeyboardModifiers state);
+
     //! This signal is emitted whenever the user attempts to modify read-only
     //! text.
     //!
@@ -1960,6 +2133,7 @@ private slots:
     void handleIndicatorClick(int pos, int modifiers);
     void handleIndicatorRelease(int pos, int modifiers);
     void handleMarginClick(int pos, int margin, int modifiers);
+    void handleMarginRightClick(int pos, int margin, int modifiers);
     void handleModified(int pos, int mtype, const char *text, int len,
             int added, int line, int foldNow, int foldPrev, int token,
             int annotationLinesAdded);
@@ -2101,8 +2275,4 @@ private:
     QsciScintilla &operator=(const QsciScintilla &);
 };
 
-#ifdef __APPLE__
-}
-#endif
-
 #endif
